#!/usr/bin/perl
#
# Executes remote commands over SSH. Requires
# passwordless SSH authentication.
#

use strict;
use warnings;

# Usage: sshrun <scheme> <action> <arguments>
my $app = 'sshrun';

my $schemes = {
	# Reserved scheme names that cannot be used:
	# /help/, -h, -?
	# list
	tcpdump => {
		usage => "$app tcpdump <start|stop> <pbxtra|trixbox> <server_id> [<port> <hostname>]",
		name  => "Run tcpdump on a remote host.\n",
		note  => "Start or stop tcpdump on a remote trixbox|pbxtra host (for monitoring UDP ports\n"
			. "for network testing purposes).\n"
			. "start|stop     => Action to perform\n"
			. "pbxtra|trixbox => Name of the software\n"
			. "server_id      => Server ID number\n"
			. "port           => Port to monitor traffic on\n"
			. "hostname       => Remote hostname that traffic will be coming from",
		args  => [
			'software',
			'sid',
			'port',
			'host',
		],
		actions => {
			start => "ssh \${software}\${sid} 'tcpdump -l port \${port} and host \${host} 2>>/dev/null'",
			stop  => "ssh \${software}\${sid} 'killall -9 tcpdump 2>>/dev/null'",
		},
	},
	ps => {
		usage => "$app ps grep <hostname> <grep>",
		name  => "Runs 'ps aux | grep \$grep' on a remote host.",
		note  => "Runs 'ps aux | grep \$grep' on a remote host.\n"
			. "hostname => Hostname of remote host\n"
			. "grep     => String to grep the 'ps aux' for",
		args => [
			'hostname',
			'grep',
		],
		actions => {
			'grep' => "ssh \${hostname} 'ps aux | grep \${grep}'",
		},
	},
};

# Make sure they used a valid scheme and action.
my $valid = 0;
my ($scheme,$action);
if (scalar(@ARGV) >= 1)
{
	$scheme = shift(@ARGV);
	$action = shift(@ARGV);

	# Help schemes.
	if ($scheme =~ /help/i || $scheme eq '-h' || $scheme eq '-?')
	{
		print "Usage: $app <scheme> <action> <arguments>\n\n"
			. "This command runs a single command over SSH. Run `$app list` to list\n"
			. "the possible schemes.\n";
		exit(0);
	}
	elsif ($scheme eq 'list')
	{
		print "List of supported schemes:\n";
		foreach my $sc (sort { $a cmp $b } keys %{$schemes})
		{
			print "$sc  -  $schemes->{$sc}->{name}\n";
		}
		exit(0);
	}

	if (exists $schemes->{$scheme} && exists $schemes->{$scheme}->{actions}->{$action})
	{
		$valid = 1;
	}
	elsif (exists $schemes->{$scheme})
	{
		print "Usage: $schemes->{$scheme}->{usage}\n"
			. "$schemes->{$scheme}->{note}\n";
		exit(0);
	}
}

unless ($valid)
{
	print "Usage: $app <scheme> <action> <arguments>\n"
		. "Run `$app help` if you need help.\n";
	exit(0);
}

# If valid, collect arguments.
my $args = {};
foreach my $arg (@{$schemes->{$scheme}->{args}})
{
	$args->{$arg} = shift(@ARGV);
}

# Prepare the command.
my $cmd = $schemes->{$scheme}->{actions}->{$action};
$cmd =~ s/\$\{(.+?)\}/$args->{$1}/ig;

# Run the command and immediately pipe the output.
$| = 1;
open (CMD, "$cmd |");
while (<CMD>)
{
	print;
}
close (CMD);
#my $out = `$cmd`;
#print $out;
exit(0);
